package org.yaoqiang.bpmn.editor.io.graphml;

import java.awt.Rectangle;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Element;

/**
 * GraphMLGenericNode
 * 
 * @author Shi Yaoqiang(shi_yaoqiang@yahoo.com)
 */
public class GraphMLGenericNode {

	private String configuration = "";

	private Rectangle geometry = new Rectangle();

	private String nodeLabel = "";

	private Map<String, String> properties = new HashMap<String, String>();

	private String bpmnType;

	private String eventCharacteristic;

	public GraphMLGenericNode(Element genericNodeElement) {
		this.configuration = genericNodeElement.getAttribute(GraphMLConstants.CONFIGURATION);
		
		Element geometryElement = GraphMLUtils.childsTag(genericNodeElement, GraphMLConstants.YWORKS + GraphMLConstants.GEOMETRY);
		Double x = Double.valueOf(geometryElement.getAttribute(GraphMLConstants.X));
		Double y = Double.valueOf(geometryElement.getAttribute(GraphMLConstants.Y));
		Double w = Double.valueOf(geometryElement.getAttribute(GraphMLConstants.WIDTH));
		Double h = Double.valueOf(geometryElement.getAttribute(GraphMLConstants.HEIGHT));
		geometry.setRect(x, y, w, h);

		Element labelElement = GraphMLUtils.childsTag(genericNodeElement, GraphMLConstants.YWORKS + GraphMLConstants.NODE_LABEL);
		if (labelElement != null) {
			this.nodeLabel = labelElement.getTextContent();
		}
		
		Element stylePropertiesElement = GraphMLUtils.childsTag(genericNodeElement, GraphMLConstants.YWORKS + GraphMLConstants.STYLE_PROPERTIES);
		if (stylePropertiesElement != null) {
			List<Element> propertyElements = GraphMLUtils.childsTags(stylePropertiesElement, GraphMLConstants.YWORKS + GraphMLConstants.PROPERTY);
			for (Element el : propertyElements) {
				properties.put(el.getAttribute(GraphMLConstants.NAME), el.getAttribute(GraphMLConstants.VALUE));
			}
			this.bpmnType = properties.get(GraphMLConstants.BPMN_TYPE);
			this.eventCharacteristic = properties.get(GraphMLConstants.EVENT_CHARACTERISTIC);			
		}

	}

	public boolean isEvent() {
		if (getConfiguration().equals(GraphMLConstants.CONFIGURATION_EVENT)) {
			return true;
		}
		return false;
	}

	public boolean isStartEvent() {
		if (isEvent() && getEventCharacteristic().equals(GraphMLConstants.EVENT_CHARACTERISTIC_START)) {
			return true;
		}
		return false;
	}

	public boolean isEntEvent() {
		if (isEvent() && getEventCharacteristic().equals(GraphMLConstants.EVENT_CHARACTERISTIC_END)) {
			return true;
		}
		return false;
	}

	public boolean isIntermediateCatchEvent() {
		if (isEvent() && getEventCharacteristic().equals(GraphMLConstants.EVENT_CHARACTERISTIC_INTERMEDIATE_CATCHING)) {
			return true;
		}
		return false;
	}

	public boolean isIntermediateThrowEvent() {
		if (isEvent() && getEventCharacteristic().equals(GraphMLConstants.EVENT_CHARACTERISTIC_INTERMEDIATE_THROWING)) {
			return true;
		}
		return false;
	}

	public boolean isNoneEvent() {
		if (isEvent() && getBpmnType().equals(GraphMLConstants.EVENT_TYPE_PLAIN)) {
			return true;
		}
		return false;
	}

	public boolean isMessageEvent() {
		if (isEvent() && getBpmnType().equals(GraphMLConstants.EVENT_TYPE_MESSAGE)) {
			return true;
		}
		return false;
	}

	public boolean isTimerEvent() {
		if (isEvent() && getBpmnType().equals(GraphMLConstants.EVENT_TYPE_TIMER)) {
			return true;
		}
		return false;
	}

	public boolean isErrorEvent() {
		if (isEvent() && getBpmnType().equals(GraphMLConstants.EVENT_TYPE_ERROR)) {
			return true;
		}
		return false;
	}

	public boolean isCancelEvent() {
		if (isEvent() && getBpmnType().equals(GraphMLConstants.EVENT_TYPE_CANCEL)) {
			return true;
		}
		return false;
	}

	public boolean isCompensationEvent() {
		if (isEvent() && getBpmnType().equals(GraphMLConstants.EVENT_TYPE_COMPENSATION)) {
			return true;
		}
		return false;
	}

	public boolean isConditionalEvent() {
		if (isEvent() && getBpmnType().equals(GraphMLConstants.EVENT_TYPE_CONDITIONAL)) {
			return true;
		}
		return false;
	}

	public boolean isSignalEvent() {
		if (isEvent() && getBpmnType().equals(GraphMLConstants.EVENT_TYPE_SIGNAL)) {
			return true;
		}
		return false;
	}

	public boolean isMultipleEvent() {
		if (isEvent() && getBpmnType().equals(GraphMLConstants.EVENT_TYPE_MULTIPLE)) {
			return true;
		}
		return false;
	}

	public boolean isLinkEvent() {
		if (isEvent() && getBpmnType().equals(GraphMLConstants.EVENT_TYPE_LINK)) {
			return true;
		}
		return false;
	}

	public boolean isTerminateEvent() {
		if (isEvent() && getBpmnType().equals(GraphMLConstants.EVENT_TYPE_TERMINATE)) {
			return true;
		}
		return false;
	}

	public boolean isGateway() {
		if (getConfiguration().equals(GraphMLConstants.CONFIGURATION_GATEWAY)) {
			return true;
		}
		return false;
	}

	public boolean isGatewayWithoutMarker() {
		if (isGateway() && getBpmnType().equals(GraphMLConstants.GATEWAY_TYPE_PLAIN)) {
			return true;
		}
		return false;
	}

	public boolean isExclusiveGateway() {
		if (isGateway() && getBpmnType().equals(GraphMLConstants.GATEWAY_TYPE_DATA_BASED_EXCLUSIVE)) {
			return true;
		}
		return false;
	}

	public boolean isExclusiveEventGateway() {
		if (isGateway() && getBpmnType().equals(GraphMLConstants.GATEWAY_TYPE_EVENT_BASED_EXCLUSIVE)) {
			return true;
		}
		return false;
	}

	public boolean isParallelGateway() {
		if (isGateway() && getBpmnType().equals(GraphMLConstants.GATEWAY_TYPE_PARALLEL)) {
			return true;
		}
		return false;
	}

	public boolean isInclusiveGateway() {
		if (isGateway() && getBpmnType().equals(GraphMLConstants.GATEWAY_TYPE_INCLUSIVE)) {
			return true;
		}
		return false;
	}

	public boolean isComplexGateway() {
		if (isGateway() && getBpmnType().equals(GraphMLConstants.GATEWAY_TYPE_COMPLEX)) {
			return true;
		}
		return false;
	}

	public boolean isActivity() {
		if (getConfiguration().equals(GraphMLConstants.CONFIGURATION_ACTIVITY)) {
			return true;
		}
		return false;
	}

	public boolean isTask() {
		if (isActivity() && getBpmnType().equals(GraphMLConstants.ACTIVITY_TYPE_STATELESS_TASK_PLAIN)) {
			return true;
		}
		return false;
	}
	
	public boolean isLoopTask() {
		if (isActivity() && getBpmnType().equals(GraphMLConstants.ACTIVITY_TYPE_STATELESS_TASK_LOOP)) {
			return true;
		}
		return false;
	}

	public boolean isAdHocTask() {
		if (isActivity() && getBpmnType().equals(GraphMLConstants.ACTIVITY_TYPE_STATELESS_TASK_AD_HOC)) {
			return true;
		}
		return false;
	}

	public boolean isMultiInstanceTask() {
		if (isActivity() && getBpmnType().equals(GraphMLConstants.ACTIVITY_TYPE_STATELESS_TASK_MULTIPLE)) {
			return true;
		}
		return false;
	}
	
	public boolean isSubProcess() {
		if (isActivity() && (getBpmnType().equals(GraphMLConstants.ACTIVITY_TYPE_SUBPROCESS_PLAIN) || getBpmnType().equals(GraphMLConstants.ACTIVITY_TYPE_TASK_PLAIN))) {
			return true;
		}
		return false;
	}

	public boolean isLoopSubProcess() {
		if (isActivity() && (getBpmnType().equals(GraphMLConstants.ACTIVITY_TYPE_TASK_LOOP) || getBpmnType().equals(GraphMLConstants.ACTIVITY_TYPE_SUBPROCESS_LOOP))) {
			return true;
		}
		return false;
	}

	public boolean isAdHocSubProcess() {
		if (isActivity() && (getBpmnType().equals(GraphMLConstants.ACTIVITY_TYPE_TASK_AD_HOC) || getBpmnType().equals(GraphMLConstants.ACTIVITY_TYPE_SUBPROCESS_AD_HOC))) {
			return true;
		}
		return false;
	}

	public boolean isMultiInstanceSubProcess() {
		if (isActivity() && (getBpmnType().equals(GraphMLConstants.ACTIVITY_TYPE_TASK_MULTIPLE) || getBpmnType().equals(GraphMLConstants.ACTIVITY_TYPE_SUBPROCESS_MULTIPLE))) {
			return true;
		}
		return false;
	}

	public boolean isArtifact() {
		if (getConfiguration().equals(GraphMLConstants.CONFIGURATION_ARTIFACT)) {
			return true;
		}
		return false;
	}

	public boolean isDataObject() {
		if (isArtifact() && getBpmnType().equals(GraphMLConstants.ARTIFACT_TYPE_DATA_OBJECT)) {
			return true;
		}
		return false;
	}

	public boolean isAnnotation() {
		if (isArtifact() && getBpmnType().equals(GraphMLConstants.ARTIFACT_TYPE_ANNOTATION)) {
			return true;
		}
		return false;
	}

	public boolean isGroup() {
		if (isArtifact() && getBpmnType().equals(GraphMLConstants.ARTIFACT_TYPE_GROUP)) {
			return true;
		}
		return false;
	}

	public String getConfiguration() {
		return configuration;
	}

	public String getEventCharacteristic() {
		return eventCharacteristic;
	}

	public Rectangle getGeometry() {
		return geometry;
	}

	public String getNodeLabel() {
		return nodeLabel == null ? "" : nodeLabel;
	}

	public String getBpmnType() {
		return bpmnType;
	}

}
