package org.yaoqiang.bpmn.editor.swing;

import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import org.yaoqiang.bpmn.editor.BPMNEditor;
import org.yaoqiang.bpmn.editor.action.BPMNCheckBoxMenuItem;
import org.yaoqiang.bpmn.editor.action.BPMNModelActions;
import org.yaoqiang.bpmn.editor.util.BPMNEditorUtils;
import org.yaoqiang.graph.editor.swing.BaseEditor;
import org.yaoqiang.graph.editor.swing.BaseMenuBar;

import com.mxgraph.swing.mxGraphComponent;
import com.mxgraph.util.mxResources;

/**
 * BPMNMenuBar
 * 
 * @author Shi Yaoqiang(shi_yaoqiang@yahoo.com)
 */
public class BPMNMenuBar extends BaseMenuBar {

	private static final long serialVersionUID = 1L;

	public BPMNMenuBar(BaseEditor editor) {

		populateFileMenu(editor);

		populateEditMenu(editor);

		populateViewMenu(editor);

		populateModelMenu(editor);

		populateDeploymentMenu(editor);

		populateSimulationMenu(editor);

		populateOrganizationsMenu(editor);

		populateSettingsMenu(editor);

		populateDiagramMenu(editor);

		populateHelpMenu(editor);

	}

	protected void populateFileMenu(BaseEditor editor) {
		JMenu menu = add(new JMenu(mxResources.get("file")));

		menu.add(editor.bind(mxResources.get("new"), BPMNModelActions.getAction(BPMNModelActions.NEW), "/org/yaoqiang/graph/editor/images/new.png"));
		menu.add(editor.bind(mxResources.get("openFile"), BPMNModelActions.getAction(BPMNModelActions.OPEN), "/org/yaoqiang/graph/editor/images/open.png"));
		menu.add(editor.bind(mxResources.get("reload"), BPMNModelActions.getAction(BPMNModelActions.RELOAD), "/org/yaoqiang/graph/editor/images/reload.png"));

		menu.addSeparator();

		menu.add(editor.bind(mxResources.get("save"), BPMNModelActions.getSaveAction(), "/org/yaoqiang/graph/editor/images/save.png"));
		menu.add(editor.bind(mxResources.get("saveAs"), BPMNModelActions.getSaveAsAction(), "/org/yaoqiang/graph/editor/images/save_as.png"));
		menu.add(editor.bind(mxResources.get("saveAsPNG"), BPMNModelActions.getSaveAsPNG(), "/org/yaoqiang/graph/editor/images/png.png"));
		menu.add(editor.bind(mxResources.get("saveAsODT"), BPMNModelActions.getSaveAsODT(), "/org/yaoqiang/bpmn/editor/images/odt.png"));
		menu.add(editor.bind(mxResources.get("saveAsFragment"), BPMNModelActions.getSaveAsFragment(), "/org/yaoqiang/bpmn/editor/images/save_as_fragment.png"));

		menu.addSeparator();

		populateFile2Menu(editor, menu);
	}

	protected void populateRecentFileMenu(BaseEditor editor) {
		BPMNEditorUtils.initRecentFileList(editor);
	}

	protected void populateGridMenu(BaseEditor editor, JMenu menu) {
		JMenu submenu = (JMenu) menu.add(new JMenu(mxResources.get("gridstyle")));
		submenu.setIcon(new ImageIcon(BPMNMenuBar.class.getResource("/org/yaoqiang/graph/editor/images/grid.png")));

		submenu.add(editor.bind(mxResources.get("gridSize"), BPMNModelActions.getAction(BPMNModelActions.GRID_SIZE)));
		submenu.add(editor.bind(mxResources.get("gridColor"), BPMNModelActions.getAction(BPMNModelActions.GRID_COLOR)));

		submenu.addSeparator();

		submenu.add(editor.bind(mxResources.get("dot"), BPMNModelActions.getGridStyleAction(mxGraphComponent.GRID_STYLE_DOT)));
		submenu.add(editor.bind(mxResources.get("cross"), BPMNModelActions.getGridStyleAction(mxGraphComponent.GRID_STYLE_CROSS)));
		submenu.add(editor.bind(mxResources.get("line"), BPMNModelActions.getGridStyleAction(mxGraphComponent.GRID_STYLE_LINE)));
		submenu.add(editor.bind(mxResources.get("dashed"), BPMNModelActions.getGridStyleAction(mxGraphComponent.GRID_STYLE_DASHED)));

		menu.addSeparator();
	}

	protected void populateModelMenu(BaseEditor editor) {
		JMenu menu = add(new JMenu(mxResources.get("model")));
		menu.add(editor.bind(mxResources.get("Definitions"), BPMNModelActions.getAction(BPMNModelActions.DEFINITIONS)));
		menu.add(editor.bind(mxResources.get("Namespaces"), BPMNModelActions.getAction(BPMNModelActions.NAMESPACES)));
		menu.add(editor.bind(mxResources.get("Imports"), BPMNModelActions.getAction(BPMNModelActions.IMPORTS)));
		menu.add(editor.bind(mxResources.get("ItemDefinitions"), BPMNModelActions.getAction(BPMNModelActions.ITEM_DEFINITIONS)));
		menu.add(editor.bind(mxResources.get("Resources"), BPMNModelActions.getAction(BPMNModelActions.RESOURCES)));
		menu.addSeparator();
		menu.add(editor.bind(mxResources.get("Messages"), BPMNModelActions.getAction(BPMNModelActions.MESSAGES)));
		menu.add(editor.bind(mxResources.get("Errors"), BPMNModelActions.getAction(BPMNModelActions.ERRORS)));
		menu.add(editor.bind(mxResources.get("Signals"), BPMNModelActions.getAction(BPMNModelActions.SIGNALS)));
		menu.add(editor.bind(mxResources.get("Escalations"), BPMNModelActions.getAction(BPMNModelActions.ESCALATIONS)));
		menu.add(editor.bind(mxResources.get("Interfaces"), BPMNModelActions.getAction(BPMNModelActions.INTERFACES)));
		menu.add(editor.bind(mxResources.get("EventDefinitions"), BPMNModelActions.getAction(BPMNModelActions.DEF_EVENT_DEFINITIONS)));
		menu.add(editor.bind(mxResources.get("Partners"), BPMNModelActions.getAction(BPMNModelActions.PARTNERS)));
		menu.addSeparator();
		menu.add(editor.bind(mxResources.get("DataStores"), BPMNModelActions.getAction(BPMNModelActions.DATASTORES)));
		menu.add(editor.bind(mxResources.get("Categories"), BPMNModelActions.getAction(BPMNModelActions.CATEGORIES)));

	}

	protected void populateDeploymentMenu(BaseEditor editor) {
		JMenu menu = add(new JMenu(mxResources.get("deployment")));
		menu.add(editor.bind(mxResources.get("DeployProcessDefinition"), BPMNModelActions.getAction(BPMNModelActions.DEPLOY),
				"/org/yaoqiang/bpmn/editor/images/deploy.png"));
		menu.add(editor.bind(mxResources.get("EngineConnections"), BPMNModelActions.getAction(BPMNModelActions.ENGINE_CONNECTIONS),
				"/org/yaoqiang/bpmn/editor/images/engine_connections.png"));
	}

	protected void populateSimulationMenu(BaseEditor editor) {
		JMenu menu = add(new JMenu(mxResources.get("simulation")));
		menu.add(editor.bind(mxResources.get("run"), BPMNModelActions.getAction(BPMNModelActions.RUN_SIMULATION), "/org/yaoqiang/bpmn/editor/images/run.png"));
		menu.add(editor.bind(mxResources.get("pause"), BPMNModelActions.getAction(BPMNModelActions.PAUSE_SIMULATION),
				"/org/yaoqiang/bpmn/editor/images/pause.png"));
		menu.add(editor.bind(mxResources.get("stop"), BPMNModelActions.getAction(BPMNModelActions.STOP_SIMULATION), "/org/yaoqiang/bpmn/editor/images/stop.png"));
		menu.addSeparator();
		menu.add(editor.bind(mxResources.get("Speed"), BPMNModelActions.getAction(BPMNModelActions.SIMULATION_SPEED),
				"/org/yaoqiang/bpmn/editor/images/timer.png"));
	}

	protected void populateOrganizationsMenu(BaseEditor editor) {
		JMenu menu = add(new JMenu(mxResources.get("organization")));
		// menu.add(editor.bind(mxResources.get("Users"), BPMNModelActions.getAction(BPMNModelActions.USERS),
		// "/org/yaoqiang/bpmn/editor/images/users.png"));
		menu.add(editor.bind(mxResources.get("Organizations"), BPMNModelActions.getAction(BPMNModelActions.ORGANIZATIONS),
				"/org/yaoqiang/bpmn/editor/images/organization.png"));
		menu.add(editor.bind(mxResources.get("LdapConnections"), BPMNModelActions.getAction(BPMNModelActions.LDAP_CONNECTIONS),
				"/org/yaoqiang/bpmn/editor/images/ldap_connections.png"));
	}

	protected void populateSettingsMenu(BaseEditor editor) {
		JMenu menu = add(new JMenu(mxResources.get("settings")));
		populateDefaultSettingsMenu(editor, menu);

		JMenu submenu = (JMenu) menu.add(new JMenu(mxResources.get("addons")));
		submenu.setIcon(new ImageIcon(BPMNMenuBar.class.getResource("/org/yaoqiang/bpmn/editor/images/addons.png")));

		populateAddonsMenu(editor, submenu);
		populateLanguageMenu(editor, menu);
		populateThemeMenu(editor, menu);
	}

	protected void populateAddonsMenu(BaseEditor editor, JMenu menu) {
		List<String> names = ((BPMNEditor) editor).getAddonsManager().getAddonInfos();
		if (names.isEmpty()) {
			menu.add(new JMenuItem(mxResources.get("none")));
		} else {
			for (String addon : names) {
				menu.add(new JMenuItem(addon));
			}
		}
	}

	protected void populateDiagramMenu(BaseEditor editor) {
		JMenu menu = add(new JMenu(mxResources.get("diagram")));
		((BPMNEditor) editor).setDiagramsMenu(menu);
		BPMNEditorUtils.refreshDiagramList((BPMNEditor) editor, null);
	}

	protected void setRulerMenuItem(BaseEditor editor) {
		rulersMenuItem = new BPMNCheckBoxMenuItem((BPMNEditor) editor, mxResources.get("rulers"), "showRulers");
	}

	protected void setGridMenuItem(BaseEditor editor) {
		gridMenuItem = new BPMNCheckBoxMenuItem((BPMNEditor) editor, mxResources.get("grid"), "showGrid");
	}
}
