package org.yaoqiang.bpmn.editor.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Rectangle;
import java.util.EventObject;

import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.text.JTextComponent;

import org.yaoqiang.bpmn.editor.swing.BPMNGraphComponent;
import org.yaoqiang.bpmn.model.BPMNModelUtils;
import org.yaoqiang.bpmn.model.elements.core.common.FlowElements;

import com.mxgraph.model.mxCell;
import com.mxgraph.model.mxIGraphModel;
import com.mxgraph.swing.mxGraphComponent;
import com.mxgraph.swing.view.mxCellEditor;
import com.mxgraph.util.mxConstants;
import com.mxgraph.util.mxUtils;
import com.mxgraph.view.mxCellState;

public class CellEditor extends mxCellEditor {

	protected transient JComboBox comboBox;

	public CellEditor(mxGraphComponent graphComponent) {
		super(graphComponent);
	}

	public BPMNGraphComponent getGraphComponent() {
		return (BPMNGraphComponent) graphComponent;
	}
	
	public Component getEditor()
	{
		if (textArea.getParent() != null)
		{
			return textArea;
		}
		// ==============start==============
		else if (comboBox != null && comboBox.getParent() != null)
		{
			return comboBox;
		}
		// ==============end================
		else if (editingCell != null)
		{
			return editorPane;
		}

		return null;
	}
	
	public Rectangle getEditorBounds(mxCellState state, double scale)
	{
		BPMNGraph graph = (BPMNGraph) state.getView().getGraph();
		mxIGraphModel model = graph.getModel();
		Rectangle bounds = null;

		if (useLabelBounds(state))
		{
			bounds = state.getLabelBounds().getRectangle();
			bounds.height += 10;
		}
		else
		{
			bounds = state.getRectangle();
		}

		// Applies the horizontal and vertical label positions
		if (model.isVertex(state.getCell()))
		{
			String horizontal = mxUtils.getString(state.getStyle(),
					mxConstants.STYLE_LABEL_POSITION, mxConstants.ALIGN_CENTER);

			if (horizontal.equals(mxConstants.ALIGN_LEFT))
			{
				bounds.x -= state.getWidth();
			}
			else if (horizontal.equals(mxConstants.ALIGN_RIGHT))
			{
				bounds.x += state.getWidth();
			}

			String vertical = mxUtils.getString(state.getStyle(),
					mxConstants.STYLE_VERTICAL_LABEL_POSITION,
					mxConstants.ALIGN_MIDDLE);

			if (vertical.equals(mxConstants.ALIGN_TOP))
			{
				bounds.y -= state.getHeight();
			}
			else if (vertical.equals(mxConstants.ALIGN_BOTTOM))
			{
				bounds.y += state.getHeight();
			}
		}

		bounds.setSize(
				(int) Math.max(bounds.getWidth(),
						Math.round(minimumWidth * scale)),
				(int) Math.max(bounds.getHeight(),
						Math.round(minimumHeight * scale)));
		// ==============start==============
		if (graph.isGroupArtifact(state.getCell()) || graph.isDataStore(state.getCell()) || graph.isDataObject(state.getCell()) && !graph.isDataInput(state.getCell()) && !graph.isDataOutput(state.getCell())) {
			int width = (comboBox == null ? 120 : comboBox.getWidth());
			int y = (int) (state.getY() + state.getHeight());
			if (graph.isGroupArtifact(state.getCell())) {
				y = (int) state.getY();
			}
			bounds.setLocation((int) state.getCenterX() - width / 2, y);
			bounds.setSize(width, (int) (25 * scale));
		}
		// ==============end================
		return bounds;
	}
	
	public void startEditing(Object cell, EventObject evt)
	{
		if (editingCell != null)
		{
			stopEditing(true);
		}

		mxCellState state = graphComponent.getGraph().getView().getState(cell);

		if (state != null)
		{
			editingCell = cell;
			trigger = evt;
			BPMNGraph graph = getGraphComponent().getGraph();
			double scale = Math.max(minimumEditorScale, graph.getView().getScale());
			scrollPane.setBounds(getEditorBounds(state, scale));
			scrollPane.setVisible(true);

			String value = getInitialValue(state, evt);
			JTextComponent currentEditor = null;

			// Configures the style of the in-place editor
			if (graph.isHtmlLabel(cell))
			{
				if (isExtractHtmlBody())
				{
					value = mxUtils.getBodyMarkup(value,
							isReplaceHtmlLinefeeds());
				}

				editorPane.setDocument(mxUtils.createHtmlDocumentObject(
						state.getStyle(), scale));
				editorPane.setText(value);

				// Workaround for wordwrapping in editor pane
				// FIXME: Cursor not visible at end of line
				JPanel wrapper = new JPanel(new BorderLayout());
				wrapper.setOpaque(false);
				wrapper.add(editorPane, BorderLayout.CENTER);
				scrollPane.setViewportView(wrapper);

				currentEditor = editorPane;
			}
			// ==============start==============
			else if (graph.getBpmnElementMap().size() > 0 && (graph.isGroupArtifact(cell) || graph.isDataStore(cell) || graph.isDataObject(state.getCell()) && !graph.isDataInput(state.getCell()) && !graph.isDataOutput(state.getCell())))
			{
				if (graph.isGroupArtifact(cell)) {
					comboBox = new JComboBox(BPMNModelUtils.getAllCategoryValues(graph.getBpmnModel()));
				} else if (graph.isDataStore(cell)) {
					comboBox = new JComboBox(BPMNModelUtils.getAllDataStores(graph.getBpmnModel()));
				} else {
					FlowElements flowElements = (FlowElements) graph.getBpmnElementMap().get(((mxCell)cell).getId()).getParent();
					if (graph.isCollectionDataObject(cell)) {
						comboBox = new JComboBox(flowElements.getAccessibleDataObjectNames(true).toArray());
					} else {
						comboBox = new JComboBox(flowElements.getAccessibleDataObjectNames(false).toArray());
					}
				}
				comboBox.setFont(mxUtils.getFont(state.getStyle(), scale));
				Color fontColor = mxUtils.getColor(state.getStyle(),
						mxConstants.STYLE_FONTCOLOR, Color.black);
				comboBox.setForeground(fontColor);
				comboBox.setSelectedItem(value);
				scrollPane.setViewportView(comboBox);
			}// ==============end================
			else
			{
				textArea.setFont(mxUtils.getFont(state.getStyle(), scale));
				Color fontColor = mxUtils.getColor(state.getStyle(),
						mxConstants.STYLE_FONTCOLOR, Color.black);
				textArea.setForeground(fontColor);
				textArea.setText(value);

				scrollPane.setViewportView(textArea);
				currentEditor = textArea;
			}

			graphComponent.getGraphControl().add(scrollPane, 0);

			if (isHideLabel(state))
			{
				graphComponent.redraw(state);
			}
			
			if (currentEditor != null) {
				currentEditor.revalidate();
				currentEditor.requestFocusInWindow();
				currentEditor.selectAll();
			}

			configureActionMaps();
		}
	}

	public void stopEditing(boolean cancel)
	{
		if (editingCell != null)
		{
			BPMNGraph graph = getGraphComponent().getGraph();
			scrollPane.transferFocusUpCycle();
			Object cell = editingCell;
			editingCell = null;

			if (!cancel)
			{
				EventObject trig = trigger;
				trigger = null;
				String currentValue = getCurrentValue();
				// ==============start==============
				if (graph.isDataObject(cell) && !graph.isDataInput(cell) && !graph.isDataOutput(cell)) {
					String oldValue = graph.convertValueToString(cell);
					if (oldValue.length() != 0) {
						int index = oldValue.lastIndexOf("\n[");
						if (index > 0) {
							currentValue += oldValue.substring(index);
						}
					}
				}
				// ==============end================
				graphComponent.labelChanged(cell, currentValue, trig);
			}
			else
			{
				mxCellState state = graph.getView()
						.getState(cell);
				graphComponent.redraw(state);
			}

			if (scrollPane.getParent() != null)
			{
				scrollPane.setVisible(false);
				scrollPane.getParent().remove(scrollPane);
			}

			graphComponent.requestFocusInWindow();
		}
	}

	protected String getInitialValue(mxCellState state, EventObject trigger)
	{
		String value = graphComponent.getEditingValue(state.getCell(), trigger);
		// ==============start==============
		BPMNGraph graph = getGraphComponent().getGraph();
		if (graph.isDataObject(state.getCell()) && !graph.isDataInput(state.getCell()) && !graph.isDataOutput(state.getCell())) {
			int index = value.lastIndexOf("\n[");
			if (index > 0) {
				return value.substring(0, index);
			}
		}
		// ==============end================
		return value;
	}
	
	public String getCurrentValue()
	{
		String result;

		if (textArea.getParent() != null)
		{
			result = textArea.getText();
		}// ==============start==============
		else if (comboBox != null && comboBox.getParent() != null) {
			if (comboBox.getSelectedItem() != null) {
				result = comboBox.getSelectedItem().toString();
			} else {
				result = "";
			}
		}// ==============end================
		else
		{
			result = editorPane.getText();

			if (isExtractHtmlBody())
			{
				result = mxUtils
						.getBodyMarkup(result, isReplaceHtmlLinefeeds());
			}
		}

		return result;
	}
}
