package org.yaoqiang.bpmn.model.elements.core.common;

import java.util.ArrayList;
import java.util.List;

import org.yaoqiang.bpmn.model.elements.XMLElement;
import org.yaoqiang.bpmn.model.elements.XMLTextElements;

/**
 * FlowNode
 * 
 * @author Shi Yaoqiang(shi_yaoqiang@yahoo.com)
 */
public abstract class FlowNode extends FlowElement {

	public FlowNode(FlowElements parent, String name) {
		super(parent, name);
	}

	protected void fillStructure() {
		XMLTextElements refIncoming = new XMLTextElements(this, "incoming");
		XMLTextElements refOutgoing = new XMLTextElements(this, "outgoing");

		super.fillStructure();
		add(refIncoming);
		add(refOutgoing);
	}

	public final List<SequenceFlow> getIncomingSequenceFlows() {
		List<SequenceFlow> sfList = new ArrayList<SequenceFlow>();
		for (XMLElement incoming : getIncomings().getXMLElements()) {
			sfList.add((SequenceFlow) getParent().getFlowElement(incoming.toValue()));
		}
		sfList.remove(null);
		return sfList;
	}

	public final List<SequenceFlow> getOutgoingSequenceFlows() {
		List<SequenceFlow> sfList = new ArrayList<SequenceFlow>();
		for (XMLElement outgoing : getOutgoings().getXMLElements()) {
			sfList.add((SequenceFlow) getParent().getFlowElement(outgoing.toValue()));
		}
		sfList.remove(null);
		return sfList;
	}

	public final XMLTextElements getIncomings() {
		return (XMLTextElements) get("incoming");
	}

	public final XMLTextElements getOutgoings() {
		return (XMLTextElements) get("outgoing");
	}

	public final void addIncoming(String incomingValue) {
		XMLElement incoming = getIncomings().generateNewElement();
		incoming.setValue(incomingValue);
		getIncomings().add(incoming);
	}

	public final void addOutgoing(String outgoingValue) {
		XMLElement outgoing = getOutgoings().generateNewElement();
		outgoing.setValue(outgoingValue);
		getOutgoings().add(outgoing);
	}

	public void removeOutgoing(String value) {
		getOutgoings().remove(value);
	}

	public void removeIncoming(String value) {
		getIncomings().remove(value);
	}

}
