package org.yaoqiang.graph.editor.swing;

import java.util.Hashtable;

import javax.swing.TransferHandler;

import org.yaoqiang.graph.editor.handler.GraphTransferHandler;
import org.yaoqiang.graph.editor.view.Graph;
import org.yaoqiang.graph.swing.YGraphComponent;
import org.yaoqiang.graph.util.Constants;

import com.mxgraph.model.mxIGraphModel;
import com.mxgraph.view.mxGraphView;

public class GraphComponent extends YGraphComponent {

	private static final long serialVersionUID = 1L;

	protected Object lastViewRoot;

	public GraphComponent(Graph graph) {
		super(graph);
	}

	public Graph getGraph() {
		return (Graph) graph;
	}

	public Object getLastViewRoot() {
		return lastViewRoot;
	}

	public void setLastViewRoot(Object lastViewRoot) {
		this.lastViewRoot = lastViewRoot;
	}

	protected TransferHandler createTransferHandler() {
		return new GraphTransferHandler();
	}

	public String validateGraph(Object cell, Hashtable<Object, Object> context) {
		mxIGraphModel model = graph.getModel();
		mxGraphView view = graph.getView();
		boolean isValid = true;
		int childCount = model.getChildCount(cell);

		for (int i = 0; i < childCount; i++) {
			Object tmp = model.getChildAt(cell, i);
			Hashtable<Object, Object> ctx = context;

			if (graph.isValidRoot(tmp)) {
				ctx = new Hashtable<Object, Object>();
			}

			String warn = validateGraph(tmp, ctx);
			// ==============start==============
			if (Constants.SETTINGS.getProperty("showWarning", "1").equals("0")) {
				setCellWarning(tmp, null);
				// setCellAttachmentIcon(tmp);
				continue;
			}
			// ==============end================
			if (warn != null) {
				String html = warn.replaceAll("\n", "<br>");
				int len = html.length();
				setCellWarning(tmp, html.substring(0, Math.max(0, len - 4)));
			} else {
				setCellWarning(tmp, null);
			}
			// setCellAttachmentIcon(tmp);// ==============start -- end==============
			isValid = isValid && warn == null;
		}

		// ==============start==============
		if (Constants.SETTINGS.getProperty("showWarning", "1").equals("0")) {
			setCellWarning(cell, null);
			// setCellAttachmentIcon(cell);
			return null;
		}
		// ==============end================
		StringBuffer warning = new StringBuffer();

		// Adds error for invalid children if collapsed (children invisible)
		// if (graph.isCellCollapsed(cell) && !isValid) {
		// warning.append(mxResources.get("containsValidationErrors", "Contains Validation Errors") + "\n");
		// }

		// Checks edges and cells using the defined multiplicities
		if (model.isEdge(cell)) {
			String tmp = graph.getEdgeValidationError(cell, model.getTerminal(cell, true), model.getTerminal(cell, false));

			if (tmp != null) {
				warning.append(tmp);
			}
		} else {
			String tmp = graph.getCellValidationError(cell);

			if (tmp != null) {
				warning.append(tmp);
			}
		}

		// Checks custom validation rules
		String err = graph.validateCell(cell, context);

		if (err != null) {
			warning.append(err);
		}

		// Updates the display with the warning icons before any potential
		// alerts are displayed
		if (model.getParent(cell) == null) {
			view.validate();
		}

		return (warning.length() > 0 || !isValid) ? warning.toString() : null;
	}

}
