package org.yaoqiang.graph.editor.view;

import org.yaoqiang.graph.editor.GraphEditor;
import org.yaoqiang.graph.editor.swing.GraphComponent;
import org.yaoqiang.graph.model.YGraphModel;

import com.mxgraph.model.mxCell;
import com.mxgraph.model.mxGraphModel;
import com.mxgraph.util.mxEvent;
import com.mxgraph.util.mxEventObject;
import com.mxgraph.util.mxEventSource;
import com.mxgraph.util.mxUndoableEdit;
import com.mxgraph.util.mxUndoableEdit.mxUndoableChange;

/**
 * GraphManager
 * 
 * @author Shi Yaoqiang(shi_yaoqiang@yahoo.com)
 */
public class GraphManager extends mxEventSource {

	protected GraphEditor editor;
	
	protected GraphComponent graphComponent;

	protected Graph graph;

	protected YGraphModel model;
	
	public GraphManager(GraphEditor editor) {
		if (this.graph != null) {
			this.graph.removeListener(handler);
		}
		this.editor = editor;
		this.graphComponent = editor.getCurrentGraphComponent();
		this.graph = graphComponent.getGraph();
		this.model = graphComponent.getGraph().getModel();

		if (this.graph != null) {
			this.graph.addListener(mxEvent.LABEL_CHANGED, handler);
			this.graph.addListener(mxEvent.CELLS_REMOVED, handler);
		}
	}

	public final mxIEventListener getHandler() {
		return handler;
	}
	
	protected mxIEventListener handler = new mxIEventListener() {
		public void invoke(Object source, mxEventObject evt) {
			String eventName = evt.getName();
			Object[] cells = (Object[]) evt.getProperty("cells");
			mxCell cell = (mxCell) graph.getSelectionCell();
			if (cell == null) {
				if (cells != null && cells.length > 0) {
					cell = (mxCell) cells[0];
				}
			}
			
			model.beginUpdate();
			try {
				if (eventName.equals(mxEvent.LABEL_CHANGED)) {
					if (graph.isCallActivityProcess(cell) && editor.getGraphComponent(cell.getId()) != null) {
						editor.getCenterPane().setTabTitle(cell.getId(), (String) cell.getValue());
					}
				} else if (eventName.equals(mxEvent.CELLS_REMOVED)) { 
					for (Object c : cells) {
						if (graph.isCallActivityProcess(c)) {
							GraphComponent graphComponent = editor.getGraphComponent(((mxCell) c).getId());
							editor.getCenterPane().remove(graphComponent);
						}
					}
				} else if (eventName.equals(mxEvent.UNDO) || eventName.equals(mxEvent.REDO)) {
					mxUndoableEdit edit = (mxUndoableEdit) evt.getProperty("edit");
					if (edit != null) {
						for (mxUndoableChange change : edit.getChanges()) {
							if (change instanceof mxGraphModel.mxValueChange) {
								mxCell c = (mxCell) ((mxGraphModel.mxValueChange) change).getCell();
								if (graph.isCallActivityProcess(c) && editor.getGraphComponent(c.getId()) != null) {
									editor.getCenterPane().setTabTitle(c.getId(), (String) c.getValue());
								}
							}
						}
					}
				}
			} finally {
				model.endUpdate();
			}
		}
		
	};
	
}
